@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 Chris Vine
+/* Copyright (C) 2001 2002 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -22,8 +22,16 @@ COPYRIGHT distributed with the source files.
 #include <cstring>
 #include <cstdlib>
 
+#include <glibmm/main.h>
+#include <glibmm/convert.h>
+#include <glibmm/timer.h>
+
 #include "efax_controller.h"
 
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#endif
+
 static pid_t child_pid;
 
 static volatile sig_atomic_t state = EfaxController::inactive;
@@ -33,11 +41,17 @@ static volatile sig_atomic_t change_state_flag = false;
 EfaxController::EfaxController(void) {
   // set up state_messages
 
-  state_messages.push_back("Inactive");
-  state_messages.push_back("Sending fax");
-  state_messages.push_back("Answering call");
-  state_messages.push_back("Answering call");
-  state_messages.push_back("Standing by to receive calls");
+  // it's OK to use std::string rather than Glib::ustring here, because
+  // the contents are passed to StatusLine::write_status(const char*) and
+  // std::string is just a transparent holder of the UTF-8 string
+  // until it reaches that method
+
+  state_messages.push_back(gettext("Inactive"));
+  state_messages.push_back(gettext("Sending fax"));
+  state_messages.push_back(gettext("Answering call"));
+  state_messages.push_back(gettext("Answering call"));
+  state_messages.push_back(gettext("Standing by to receive calls"));
+  state_messages.push_back("");
   state_messages.push_back("");
   state_messages.push_back("");
   state_messages.push_back("");
@@ -48,7 +62,7 @@ void EfaxController::efax_closedown(void) {
   if (state != inactive) {
     state = closedown;
     kill_child();
-    while (state != inactive) usleep(500000);
+    while (state != inactive) Glib::usleep(500000);
     if (*prog_config.receive_dirname) {
       // now delete prog_config.receive_dirname if it is empty
       string full_dirname;
@@ -110,8 +124,10 @@ void EfaxController::sendfax(string& file, string& number) {
 
       string temp("-h");
       temp += date_string;
-      temp += "    From ";
-      temp += prog_config.my_name + " (";
+      temp += "    ";
+      temp += Glib::locale_from_utf8(gettext("From"));
+      temp += " ";
+      temp += Glib::locale_from_utf8(prog_config.my_name) + " (";
       temp += prog_config.my_number + ") --> ";
       temp += number + "    p.%d/%d";
       prog_config.parms.push_back(temp);
@@ -147,7 +163,7 @@ void EfaxController::sendfax(string& file, string& number) {
 	            "and the PATH environmental variable\n");
       }
       // this child process must end here - use _exit() not exit()
-      usleep(200000);
+      Glib::usleep(200000);
       _exit(EXEC_ERROR); 
     } // end of child process
 
@@ -168,9 +184,17 @@ vector<string> EfaxController::make_fax(string filename) {
   string::size_type pos = filename.find_last_of('/');
 
   if (pos == string::npos || pos + 2 > filename.size()) {
-    write_error("Not valid file name\n");
+    write_error(Glib::locale_from_utf8(gettext("Not valid file name\n")).c_str());
   }
     
+  else if (access(filename.c_str(), F_OK)) {
+    write_error(Glib::locale_from_utf8(gettext("File does not exist\n")).c_str());
+  }
+
+  else if (access(filename.c_str(), R_OK)) {
+    write_error(Glib::locale_from_utf8(gettext("User does not have read permission on the file\n")).c_str());
+  }
+
   else {
 
     pid_t pid = fork();
@@ -240,7 +264,7 @@ vector<string> EfaxController::make_fax(string filename) {
       write_error("Can't find the ghostscript program - please check your installation\n"
 		  "and the PATH environmental variable\n");
       // this child process must end here - use _exit() not exit()
-      usleep(200000);
+      Glib::usleep(200000);
       _exit(EXEC_ERROR); 
     } // end of child process
 
@@ -267,7 +291,7 @@ vector<string> EfaxController::make_fax(string filename) {
     }
     delete[] test_name;
     
-    if (vec.empty()) write_error("Not valid postscript file\n");
+    if (vec.empty()) write_error(Glib::locale_from_utf8(gettext("Not valid postscript file\n")).c_str());
   }
 
   return vec;
@@ -379,7 +403,7 @@ void EfaxController::receive(int mode) {
       write_error("Can't find the efax program - please check your installation\n"
 		  "and the PATH environmental variable\n");
       // this child process must end here - use _exit() not exit()
-      usleep(200000);
+      Glib::usleep(200000);
       _exit(EXEC_ERROR); 
     } // end of child process
 
@@ -391,16 +415,15 @@ void EfaxController::receive(int mode) {
   }
 }
 
-void EfaxController::read_pipe_slot(gint fd, GdkInputCondition) {
+bool EfaxController::read_pipe_slot(Glib::IOCondition) {
   char pipe_buffer[PIPE_BUF + 1];
   ssize_t result;
 
-  if (fd == stdout_pipe.get_read_fd()) {
-    while ((result = stdout_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
-      pipe_buffer[result] = 0;
-      stdout_message(pipe_buffer);
-    }
+  while ((result = stdout_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
+    pipe_buffer[result] = 0;
+    stdout_message(pipe_buffer);
   }
+  return true; // retain connection for further reads
 }
 
 void EfaxController::stop_slot(void) {
@@ -440,8 +463,9 @@ void EfaxController::timer_event(void) {
       if (state_val == restart_standby) receive(receive_standby);
     }
 
-    else if (state == send_success) {
-      save_sent_fax();
+    else if (state == send_success || state == send_fail) {
+      if (state == send_success) save_sent_fax();
+      else cleanup_fax_send_fail();
       unjoin_child();
       state = inactive;
     }
@@ -456,8 +480,8 @@ void EfaxController::display_state(void) {
 }
 
 void EfaxController::join_child(void) {
-  stdout_connection = Gtk::Main::input.connect(SigC::slot(this, &EfaxController::read_pipe_slot),
-					       stdout_pipe.get_read_fd(), GDK_INPUT_READ);
+  stdout_connection = Glib::signal_io().connect(SigC::slot(*this, &EfaxController::read_pipe_slot),
+						stdout_pipe.get_read_fd(), Glib::IO_IN);
 }
 
 void EfaxController::unjoin_child(void) {
@@ -467,7 +491,7 @@ void EfaxController::unjoin_child(void) {
 
 void EfaxController::kill_child(void) {
   if (child_pid) kill(child_pid, SIGTERM);
-  usleep(500000);               // now really make sure
+  Glib::usleep(500000);               // now really make sure
   if (child_pid) kill(child_pid, SIGKILL);
 }
 
@@ -567,10 +591,11 @@ void EfaxController::save_sent_fax(void) {
 	filein.read(block, BLOCKSIZE);
 	fileout.write(block, filein.gcount());
       }
-      filein.close();
       filein.clear();
-      fileout.close();
+      filein.close();
       fileout.clear();
+      fileout.close();
+      unlink(suffixed_inname.c_str());
       
       partnumber++;
       ostrstream strm;
@@ -591,18 +616,46 @@ void EfaxController::save_sent_fax(void) {
   }
 }
 
+void EfaxController::cleanup_fax_send_fail(void) {
+
+  // make the suffix
+  int partnumber = 1;
+  ostrstream strm;
+  strm << '.' << setfill('0') << setw(3) << partnumber << ends;
+  const char* suffix = strm.str();
+
+  // make the suffixed source and destination files
+  string filename = last_file_sent + suffix;
+
+  delete[] suffix;
+
+  while (!access(filename.c_str(), R_OK)) {
+    unlink(filename.c_str());
+      
+    partnumber++;
+    ostrstream strm;
+    strm << '.' << setfill('0') << setw(3) << partnumber << ends;
+    suffix = strm.str();
+    filename = last_file_sent + suffix;
+      
+    delete[] suffix;
+  }
+}
+
 bool efax_controller_childexit_handler(pid_t pid, int exit_code) {
   if (pid == child_pid) {
     child_pid = 0;
-    unlink(prog_config.lock_file.c_str());  // this won't work if efax is suid root and we are not root
+    unlink(Glib::filename_from_utf8(prog_config.lock_file).c_str());  // this won't work if efax is suid root and we are not root
     if (state == EfaxController::closedown) // tell EfaxController::efax_closedown() the child is now dead
       state = EfaxController::inactive;
     else if (state == EfaxController::receive_standby
 	     && (!exit_code || exit_code == 3)) // get EfaxController::timer_event() to restart efax()
       state = EfaxController::restart_standby;
-    else if (state == EfaxController::sending
-	     && !exit_code)            // get EfaxController::timer_event() to save the file in sent fax list
-      state = EfaxController::send_success;
+    else if (state == EfaxController::sending) {
+      if(!exit_code)            // get EfaxController::timer_event() to save the file in sent fax list
+	state = EfaxController::send_success;
+      else state = EfaxController::send_fail;
+    }
     else if (state != EfaxController::inactive) // get EfaxController::timer_event() to close dead file descriptors and dead SigC connections
       state = EfaxController::close_child_connections;
 
