@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 Chris Vine
+/* Copyright (C) 2001 2002 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -17,18 +17,25 @@ COPYRIGHT distributed with the source files.
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
+#include <vector>
 
-#include <gtk--/main.h>
-#include <gtk--/menu.h>
-#include <gtk--/scrollbar.h>
-#include <gtk--/adjustment.h>
-#include <gtk--/style.h>
-#include <gdk--/color.h>
+#include <gtkmm/menu.h>
+#include <gtkmm/style.h>
+#include <gdkmm/color.h>
+#include <gdkmm/pixbuf.h>
+#include <glibmm/main.h>
+#include <glibmm/convert.h>
 
 #include "mainwindow.h"
 #include "dialogs.h"
 #include "addressbook.h"
 #include "settings.h"
+#include "helpfile.h"
+#include "window_icon.h"
+
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#endif
 
 #define TIMER_INTERVAL 200              // number of milliseconds between timer events for the main timer
 #define LOGFILE_TIMER_INTERVAL 60000    // number of milliseconds between flushing of logfile
@@ -42,11 +49,14 @@ Pipe_fifo MainWindow::error_pipe(Pipe_fifo::non_block);
 bool MainWindow::connected_to_stderr = false;
 
 MainWindow::MainWindow(const string& messages, const char* filename): // filename has a default value of 0
-                              Gtk::Window(GTK_WINDOW_TOPLEVEL), standard_size(24),
-                              table(5, 3, false), file_button("File to fax:"),
-			      number_button("Tel number: "), send_button("Send fax"),
-			      receive_answer_button("Answer call"), receive_takeover_button("Take over call"),
-			      receive_standby_button("Standby"), stop_button("Stop"),
+                              Gtk::Window(Gtk::WINDOW_TOPLEVEL), standard_size(24),
+                              table(5, 3, false), file_button(gettext("File to fax:")),
+			      number_button(gettext("Tel number: ")),
+			      send_button(gettext("Send fax")),
+			      receive_answer_button(gettext("Answer call")),
+			      receive_takeover_button(gettext("Take over call")),
+			      receive_standby_button(gettext("Standby")),
+			      stop_button(gettext("Stop")),
                               status_line(standard_size) {
 
   // catch any relevant Unix signals for an orderly closedown
@@ -63,16 +73,11 @@ MainWindow::MainWindow(const string& messages, const char* filename): // filenam
   sig_act.sa_handler = childexit_signalhandler;
   // we don't need to mask off any signals
   sigemptyset(&sig_act.sa_mask);
-  // the default flag set will be ok here
-  sig_act.sa_flags = 0;
+  sig_act.sa_flags = SA_NOCLDSTOP;
 
   sigaction(SIGCHLD, &sig_act, 0);
 
-  file_button.set_usize(standard_size * 4, standard_size);
-  number_button.set_usize(standard_size * 4, standard_size);
-
-  file_entry.set_usize(standard_size * 7, standard_size);
-  number_entry.set_usize(standard_size * 7, standard_size);
+  get_longest_button_text();
 
   file_box.pack_start(file_button, false, false, standard_size/2);
   file_box.pack_start(file_entry, true, true, 0);
@@ -80,45 +85,43 @@ MainWindow::MainWindow(const string& messages, const char* filename): // filenam
   number_box.pack_start(number_button, false, false, standard_size/2);
   number_box.pack_start(number_entry, true, true, 0);
 
-  text_window.set_usize(standard_size * 13, standard_size * 5);
+  table.attach(file_box, 0, 3, 0, 1, Gtk::EXPAND | Gtk::FILL,
+	       Gtk::SHRINK, standard_size/3, standard_size/3);
+  table.attach(number_box, 0, 3, 1, 2, Gtk::EXPAND | Gtk::FILL,
+         Gtk::SHRINK, standard_size/3, standard_size/3);
+  table.attach(text_window, 0, 3, 2, 3, Gtk::EXPAND | Gtk::FILL,
+         Gtk::EXPAND | Gtk::FILL, standard_size/3, standard_size/3);
 
-  table.attach(file_box, 0, 3, 0, 1, GTK_EXPAND | GTK_FILL,
-         0, standard_size/3, standard_size/3);
-  table.attach(number_box, 0, 3, 1, 2, GTK_EXPAND | GTK_FILL,
-         0, standard_size/3, standard_size/3);
-  table.attach(text_window, 0, 3, 2, 3, GTK_EXPAND | GTK_FILL,
-         GTK_EXPAND | GTK_FILL, standard_size/3, standard_size/3);
+  table.attach(send_button, 0, 1, 3, 4, Gtk::EXPAND,
+	 Gtk::SHRINK, standard_size/3, standard_size/3);
+  table.attach(receive_answer_button, 1, 2, 3, 4, Gtk::EXPAND,
+	 Gtk::SHRINK, standard_size/3, standard_size/3);
+  table.attach(receive_takeover_button, 2, 3, 3, 4, Gtk::EXPAND,
+	 Gtk::SHRINK, standard_size/3, standard_size/3);
 
-  table.attach(send_button, 0, 1, 3, 4, GTK_EXPAND,
-	 0, standard_size/3, standard_size/3);
-  table.attach(receive_answer_button, 1, 2, 3, 4, GTK_EXPAND,
-	 0, standard_size/3, standard_size/3);
-  table.attach(receive_takeover_button, 2, 3, 3, 4, GTK_EXPAND,
-	 0, standard_size/3, standard_size/3);
+  table.attach(receive_standby_button, 0, 1, 4, 5, Gtk::EXPAND,
+	 Gtk::SHRINK, standard_size/3, standard_size/3);
+  table.attach(stop_button, 2, 3, 4, 5, Gtk::EXPAND,
+	 Gtk::SHRINK, standard_size/3, standard_size/3);
 
-  table.attach(receive_standby_button, 0, 1, 4, 5, GTK_EXPAND,
-	 0, standard_size/3, standard_size/3);
-  table.attach(stop_button, 2, 3, 4, 5, GTK_EXPAND,
-	 0, standard_size/3, standard_size/3);
-
-  file_button.clicked.connect(SigC::slot(this, &MainWindow::get_file_slot));
-  number_button.clicked.connect(SigC::slot(this, &MainWindow::addressbook_slot));
-  send_button.clicked.connect(SigC::slot(this, &MainWindow::sendfax_slot));
-  receive_answer_button.clicked.connect(SigC::bind(SigC::slot(this, &MainWindow::receive_slot),
+  file_button.signal_clicked().connect(SigC::slot(*this, &MainWindow::get_file_slot));
+  number_button.signal_clicked().connect(SigC::slot(*this, &MainWindow::addressbook_slot));
+  send_button.signal_clicked().connect(SigC::slot(*this, &MainWindow::sendfax_slot));
+  receive_answer_button.signal_clicked().connect(SigC::bind(SigC::slot(*this, &MainWindow::receive_slot),
 						   EfaxController::receive_answer));
-  receive_takeover_button.clicked.connect(SigC::bind(SigC::slot(this, &MainWindow::receive_slot),
+  receive_takeover_button.signal_clicked().connect(SigC::bind(SigC::slot(*this, &MainWindow::receive_slot),
 						     EfaxController::receive_takeover));
-  receive_standby_button.clicked.connect(SigC::bind(SigC::slot(this, &MainWindow::receive_slot),
+  receive_standby_button.signal_clicked().connect(SigC::bind(SigC::slot(*this, &MainWindow::receive_slot),
 						    EfaxController::receive_standby));
-  stop_button.clicked.connect(SigC::slot(&efax_controller, &EfaxController::stop_slot));
+  stop_button.signal_clicked().connect(SigC::slot(efax_controller, &EfaxController::stop_slot));
 
-  destroy.connect(SigC::slot(this, &MainWindow::destroy_slot));
-
-  send_button.set_usize(standard_size * 4, standard_size);
-  receive_answer_button.set_usize(standard_size * 4, standard_size);
-  receive_takeover_button.set_usize(standard_size * 4, standard_size);
-  receive_standby_button.set_usize(standard_size * 4, standard_size);
-  stop_button.set_usize(standard_size * 4, standard_size);
+  file_button.set_flags(Gtk::CAN_DEFAULT);
+  number_button.set_flags(Gtk::CAN_DEFAULT);
+  send_button.set_flags(Gtk::CAN_DEFAULT);
+  receive_answer_button.set_flags(Gtk::CAN_DEFAULT);
+  receive_takeover_button.set_flags(Gtk::CAN_DEFAULT);
+  receive_standby_button.set_flags(Gtk::CAN_DEFAULT);
+  stop_button.set_flags(Gtk::CAN_DEFAULT);
 
   // set up the menu bar
   {
@@ -128,40 +131,41 @@ MainWindow::MainWindow(const string& messages, const char* filename): // filenam
     // now fill the file menu
     MenuList& file_list = file_menu_p->items();
     
-    file_list.push_back(MenuElem("List _received faxes",
-				 SigC::bind(SigC::slot(this, &MainWindow::fax_list_slot), 
+    file_list.push_back(MenuElem(gettext("List _received faxes"),
+				 SigC::bind(SigC::slot(*this, &MainWindow::fax_list_slot), 
 					    FaxListDialog::received)));
-    file_list.push_back(MenuElem("_List sent faxes",
-				 SigC::bind(SigC::slot(this, &MainWindow::fax_list_slot), 
+    file_list.push_back(MenuElem(gettext("_List sent faxes"),
+				 SigC::bind(SigC::slot(*this, &MainWindow::fax_list_slot), 
 					    FaxListDialog::sent)));
     file_list.push_back(SeparatorElem());
-    file_list.push_back(MenuElem("_Find file",
-				 SigC::slot(this, &MainWindow::get_file_slot)));
-    file_list.push_back(MenuElem("_Address book",
-				 SigC::slot(this, &MainWindow::addressbook_slot)));
+    file_list.push_back(MenuElem(gettext("_Find file"),
+				 SigC::slot(*this, &MainWindow::get_file_slot)));
+    file_list.push_back(MenuElem(gettext("_Address book"),
+				 SigC::slot(*this, &MainWindow::addressbook_slot)));
     file_list.push_back(SeparatorElem());
-    file_list.push_back(MenuElem("_Settings",
-				 SigC::slot(this, &MainWindow::settings_slot)));
+    file_list.push_back(MenuElem(gettext("_Settings"),
+				 SigC::slot(*this, &MainWindow::settings_slot)));
     file_list.push_back(SeparatorElem());
-    file_list.push_back(MenuElem("_Quit", destroy.slot()));
+    file_list.push_back(MenuElem(gettext("_Quit"),
+				 SigC::slot(*this, &MainWindow::quit_slot)));
 
     // Create the about menu
     Gtk::Menu* help_menu_p = manage(new Gtk::Menu);
     // now fill the about menu
     MenuList& help_list = help_menu_p->items();
     
-    help_list.push_back(MenuElem("About efax-_gtk",
-				 SigC::bind(SigC::slot(this, &MainWindow::about_slot), true)));
-    help_list.push_back(MenuElem("About _efax",
-				 SigC::bind(SigC::slot(this, &MainWindow::about_slot), false)));
+    help_list.push_back(MenuElem(gettext("About efax-_gtk"),
+				 SigC::bind(SigC::slot(*this, &MainWindow::about_slot), true)));
+    help_list.push_back(MenuElem(gettext("About _efax"),
+				 SigC::bind(SigC::slot(*this, &MainWindow::about_slot), false)));
     help_list.push_back(SeparatorElem());
-    help_list.push_back(MenuElem("_Helpfile",
-				 SigC::slot(this, &MainWindow::helpfile_slot)));
+    help_list.push_back(MenuElem(gettext("_Help"),
+				 SigC::slot(*this, &MainWindow::helpfile_slot)));
 
     // now fill the menu bar
     MenuList& bar_list = menu_bar.items();
-    bar_list.push_front(MenuElem("_Help", "<mod1>h", *help_menu_p));
-    bar_list.push_front(MenuElem("_File", "<mod1>f", *file_menu_p));
+    bar_list.push_front(MenuElem(gettext("_Help"), *help_menu_p));
+    bar_list.push_front(MenuElem(gettext("_File"), *file_menu_p));
   }
 
   window_box.pack_start(menu_bar, false, false);
@@ -172,24 +176,38 @@ MainWindow::MainWindow(const string& messages, const char* filename): // filenam
 
   table.set_border_width(standard_size/3);
 
+  std::vector<Gtk::Widget*> focus_chain;
+  focus_chain.push_back(&file_box);
+  focus_chain.push_back(&number_box);
+  focus_chain.push_back(&send_button);
+  focus_chain.push_back(&receive_answer_button);
+  focus_chain.push_back(&receive_takeover_button);
+  focus_chain.push_back(&receive_standby_button);
+  focus_chain.push_back(&stop_button);
+  table.set_focus_chain(focus_chain);
+
   if (filename) {
     file_entry.set_text(filename);
-    number_entry.grab_focus();
+    set_focus(number_entry);
   }
-  else file_button.grab_focus();
+  else set_focus(file_button);
 
-  Gtk::Main::timeout.connect(SigC::slot(this, &MainWindow::timer_event_handler), TIMER_INTERVAL);
+  Glib::signal_timeout().connect(SigC::slot(*this, &MainWindow::timer_event_handler), TIMER_INTERVAL);
+  Glib::signal_io().connect(SigC::slot(*this, &MainWindow::read_error_slot),
+			    error_pipe.get_read_fd(), Glib::IO_IN);
 
-  Gtk::Main::input.connect(SigC::slot(this, &MainWindow::read_error_slot),
-			   error_pipe.get_read_fd(), GDK_INPUT_READ);
-
-  efax_controller.stdout_message.connect(SigC::slot(&text_window, &MessageText::write_black_slot));
-  efax_controller.write_state.connect(SigC::slot(&status_line, &StatusLine::write_status));
+  efax_controller.stdout_message.connect(SigC::slot(text_window, &MessageText::write_black_slot));
+  efax_controller.write_state.connect(SigC::slot(status_line, &StatusLine::write_status));
 
   // we want to minimise the effect on efax, so make writing to the error pipe non-blocking
   error_pipe.make_write_non_block();
 
+  set_default_size(standard_size * 15, standard_size * 15);
+
+  set_icon(Gdk::Pixbuf::create_from_xpm_data(window_icon_xpm));
+
   show_all();
+
   if (!messages.empty()) {
     text_window.write_red_slot(messages.c_str());
     text_window.write_red_slot("\n\n");
@@ -212,52 +230,43 @@ MainWindow::MainWindow(const string& messages, const char* filename): // filenam
     // a configuration file will be reported twice -- not a big deal, but ...
     // so pass true as the last parameter to skip detection of settings file
     SettingsDialog dialog(standard_size, *this, true);
-    dialog.accepted.connect(SigC::slot(this, &MainWindow::settings_changed_slot));
+    dialog.accepted.connect(SigC::slot(*this, &MainWindow::settings_changed_slot));
     dialog.run();
   }
+
 }
 
-gint MainWindow::key_press_event_impl(GdkEventKey* event_p) {
+bool MainWindow::on_key_press_event(GdkEventKey* event_p) {
   if (event_p->keyval == GDK_F1) helpfile_slot();
-  else Gtk::Window::key_press_event_impl(event_p);
+  else Gtk::Window::on_key_press_event(event_p);
   return true; // processing stops here
 }
 
-void MainWindow::read_error_slot(gint fd, GdkInputCondition) {
+bool MainWindow::read_error_slot(Glib::IOCondition) {
 
   char pipe_buffer[PIPE_BUF + 1];
   ssize_t result;
 
-  if (fd == error_pipe.get_read_fd()) {
-    while ((result = error_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
-      pipe_buffer[result] = 0;
-      text_window.write_red_slot(pipe_buffer);
-    }
+  while ((result = error_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
+    pipe_buffer[result] = 0;
+    text_window.write_red_slot(pipe_buffer);
   }
+  return true; // retain connection for further reads
 }
 
-void MainWindow::destroy_slot(void) {
-  // there are a number of ways to close the program
-  // one is to call Gtk::Main::quit() and then to follow it with
-  // an explicit emission of destroy()
-  // or delete_event_impl() can call Gtk::Main::quit() and return false
-  // (which in fact just emits destroy() and so has the same
-  // effect as the first case)
-  // another (which we do here) is to call Gtk::Main::quit() within a method
-  // connected to the destroy signal, and quit the program by emitting
-  // the destroy signal
+void MainWindow::quit_slot(void) {
 
   // first, make sure we close down efax() if it is active
   efax_controller.efax_closedown();
 
   // now quit
-  Gtk::Main::quit();
+  hide(); // hiding the main program window will cause the program to terminate
 }
 
 void MainWindow::sendfax_slot(void) {
 
-  string file(file_entry.get_text());
-  string number(number_entry.get_text());
+  string file(Glib::filename_from_utf8(file_entry.get_text()));
+  string number(Glib::locale_from_utf8(number_entry.get_text()));
 
   if (!prog_config.found_rcfile) {
     text_window.write_red_slot("Can't send fax -- no efax-gtkrc configuration file found\n\n");
@@ -290,16 +299,16 @@ void MainWindow::receive_slot(int mode) {
   else efax_controller.receive(mode);
 }
 
-gint MainWindow::timer_event_handler(void) {
+bool MainWindow::timer_event_handler(void) {
 
-  if (close_flag) destroy(); // we must have picked up an external kill signal
-                             // so we need an orderly close down
+  if (close_flag) quit_slot(); // we must have picked up an external kill signal
+                               // so we need an orderly close down
   else efax_controller.timer_event();
   return true; // we want a multi-shot timer
 }
 
 void MainWindow::get_file_slot(void) {
-  FileReadSelectDialog file_dialog(standard_size, *this);
+  FileReadSelectDialog file_dialog(file_entry.get_text(), standard_size, *this);
   string result(file_dialog.run());
   if (!result.empty()) {
     file_entry.set_text(result);
@@ -331,7 +340,7 @@ void MainWindow::addressbook_slot(void) {
       cerr << "Memory allocation error in MainWindow::addressbook_slot()" << endl;
       exit(MEM_ERROR);
     }
-    dialog_p->accepted.connect(SigC::slot(this, &MainWindow::set_number));
+    dialog_p->accepted.connect(SigC::slot(*this, &MainWindow::set_number));
     // there is no memory leak -- AddressBook will delete its own memory
     // when it is closed
   }
@@ -341,10 +350,10 @@ void MainWindow::settings_slot(void) {
 
   if (efax_controller.get_state() != EfaxController::inactive) {
     InfoDialog* dialog_p;
-    const char message[] = "Can't change settings unless\n"
-                           "the program is inactive\n\n"
-                           "Press the Stop button to make it inactive\n";
-    dialog_p = new InfoDialog(message, "Change settings", standard_size, InfoDialog::warning, *this);
+    Glib::ustring message(gettext("Can't change settings unless "
+				  "the program is inactive\n"
+				  "Press the Stop button to make it inactive"));
+    dialog_p = new InfoDialog(message, "Change settings", standard_size, Gtk::MESSAGE_WARNING, *this);
     if (!dialog_p) {
       cerr << "Memory allocation error in MainWindow::settings_slot()" << endl;
       exit(MEM_ERROR);
@@ -359,16 +368,16 @@ void MainWindow::settings_slot(void) {
       cerr << "Memory allocation error in MainWindow::settings_slot()" << endl;
       exit(MEM_ERROR);
     }
-    dialog_p->accepted.connect(SigC::slot(this, &MainWindow::settings_changed_slot));
+    dialog_p->accepted.connect(SigC::slot(*this, &MainWindow::settings_changed_slot));
     // there is no memory leak -- SettingsDialog will delete its own memory
     // when it is closed
   }
 }
 
-void MainWindow::settings_changed_slot(const string& messages) {
+void MainWindow::settings_changed_slot(const Glib::ustring& messages) {
   text_window.reset_logfile();
   if (!messages.empty()) {
-    text_window.write_red_slot(messages.c_str());
+    text_window.write_red_slot(Glib::locale_from_utf8(messages).c_str());
     text_window.write_red_slot("\n\n");
   }
 }
@@ -384,7 +393,7 @@ void MainWindow::helpfile_slot(void) {
   // when it is closed
 }
 
-void MainWindow::set_number(const string& number) {
+void MainWindow::set_number(const Glib::ustring& number) {
 
   if (!number.empty()) {
     number_entry.set_text(number);
@@ -396,19 +405,19 @@ void MainWindow::about_slot(bool efax_gtk) {
   InfoDialog* dialog_p;
 
   if (efax_gtk) {
-    const char message[] = "efax-gtk-1.0.1a\n"
-                           "Copyright (C) 2001 Chris Vine\n\n"
-                           "This program is released under the\n"
-                           "GNU General Public License, version 2\n";
-    dialog_p = new InfoDialog(message, "About efax-gtk", standard_size, InfoDialog::information, *this);
+    Glib::ustring message = gettext("efax-gtk-2.0.5a\n"
+				    "Copyright (C) 2001 - 2003 Chris Vine\n\n"
+				    "This program is released under the "
+				    "GNU General Public License, version 2");
+    dialog_p = new InfoDialog(message, "About efax-gtk", standard_size, Gtk::MESSAGE_INFO, *this);
   }
   else {
-    const char message[] = "This program is a front end for efax\n\n"
-                           "efax is a program released under the\n"
-                           "GNU General Public License, version 2 by Ed Casas\n\n"
-                           "The copyright to efax is held by Ed Casas\n"
-                           "(the copyright to this program is held by Chris Vine)\n";
-    dialog_p = new InfoDialog(message, "About efax", standard_size, InfoDialog::information, *this);
+    Glib::ustring message = gettext("This program is a front end for efax.\n"
+				    "efax is a program released under the "
+				    "GNU General Public License, version 2 by Ed Casas.\n\n"
+				    "The copyright to efax is held by Ed Casas "
+				    "(the copyright to this program is held by Chris Vine)");
+    dialog_p = new InfoDialog(message, "About efax", standard_size, Gtk::MESSAGE_INFO, *this);
   }
 
   if (!dialog_p) {
@@ -419,22 +428,84 @@ void MainWindow::about_slot(bool efax_gtk) {
   // will delete its own memory when it is closed
 }
 
-MessageText::MessageText(void) : Gtk::Table(1, 2, false) {
+void MainWindow::get_longest_button_text(void) {
+  std::vector<Glib::ustring> text_vec;
+  text_vec.push_back(gettext("File to fax:"));
+  text_vec.push_back(gettext("Tel number: "));
+  text_vec.push_back(gettext("Send fax"));
+  text_vec.push_back(gettext("Answer call"));
+  text_vec.push_back(gettext("Take over call"));
+  text_vec.push_back(gettext("Standby"));
+  text_vec.push_back(gettext("Stop"));
 
-  attach(text, 0, 1, 0, 1, GTK_FILL | GTK_EXPAND,
-	 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
-  // put a VScrollbar in the upper right
-  Gtk::Scrollbar* scrollbar_p = manage(new Gtk::VScrollbar(*(text.get_vadjustment())));
-  attach (*scrollbar_p, 1, 2, 0, 1, GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
+  Gtk::Button button;
+  std::vector<Glib::ustring>::iterator temp_iter;
+  std::vector<Glib::ustring>::iterator max_width_iter;
+  int width;
+  int height;
+  int max_width;
+  for (temp_iter = text_vec.begin(), max_width_iter = text_vec.begin(),
+	 width = 0, height = 0, max_width = 0;
+       temp_iter != text_vec.end(); ++temp_iter) {
 
-  text.set_word_wrap(true);
-  text.set_line_wrap(true);
+    button.create_pango_layout(*temp_iter)->get_pixel_size(width, height);
+
+    if (width > max_width) {
+      max_width = width;
+      max_width_iter = temp_iter;
+    }
+  }
+  max_text = *max_width_iter;
+}
+
+void MainWindow::on_style_changed(const Glib::RefPtr<Gtk::Style>& prev_style) {
+
+  // before we set the buttons, do anything else necessary with the new style
+  Gtk::Window::on_style_changed(prev_style);
+
+  //now set the buttons in MainWindow
+  int width = 0;
+  int height = 0;
+  Gtk::Button button;
+
+  button.set_style(get_style());
+  button.create_pango_layout(max_text)->get_pixel_size(width, height);
+
+  // add padding
+  width += 18;
+  height += 12;
+
+  // have some sensible minimum width and height if a very small font chosen
+  const int min_width = standard_size * 4;
+  const int min_height = standard_size + 4;
+  if (width < min_width) width = min_width;
+  if (height < min_height) height = min_height;
+
+  file_button.set_size_request(width, height);
+  number_button.set_size_request(width, height);
+  send_button.set_size_request(width, height);
+  receive_answer_button.set_size_request(width, height);
+  receive_takeover_button.set_size_request(width, height);
+  receive_standby_button.set_size_request(width, height);
+  stop_button.set_size_request(width, height);
+
+  status_line.set_colour(get_style());
+}
+
+MessageText::MessageText(void) {
+
+  text.set_wrap_mode(Gtk::WRAP_WORD);
   text.set_editable(false);
-  text.unset_flags(GTK_CAN_FOCUS);
+  text.unset_flags(Gtk::CAN_FOCUS);
+  add(text);
+  set_shadow_type(Gtk::SHADOW_IN);
+  set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);
 
-  Gdk_Color red;
+  Gdk::Color red;
   red.set_rgb_p(0.8, 0, 0);
-  red_context.set_foreground(red);
+  red_tag_r = Gtk::TextTag::create();
+  red_tag_r->property_foreground_gdk() = red;
+  text.get_buffer()->get_tag_table()->add(red_tag_r);
 
   // save the current working directory before MainWindow::MainWindow can get
   // at it, in case it is needed in future by reset_logfile
@@ -458,7 +529,7 @@ MessageText::MessageText(void) : Gtk::Table(1, 2, false) {
   // now open the log file if required
   if (!prog_config.logfile_name.empty()) {
 
-    string temp(prog_config.logfile_name);
+    string temp(Glib::filename_from_utf8(prog_config.logfile_name));
     if (temp[0] != '/') temp.insert(0, starting_dirname); // provide an absolute path name
     logfile.open(temp.c_str(), ios::app | ios::out);
     if (!logfile) {
@@ -482,12 +553,12 @@ MessageText::MessageText(void) : Gtk::Table(1, 2, false) {
       logfile << "\n\n***********************\n"
                  "Beginning fax log at " << date_description << '\n' << endl;
       // set a timeout to flush every one minute
-      timer_slot_connection = Gtk::Main::timeout.connect(SigC::slot(this,
+      timer_slot_connection = Glib::signal_timeout().connect(SigC::slot(*this,
                                          &MessageText::flush_logfile_timer_slot), LOGFILE_TIMER_INTERVAL);
     }
     // save the logfile name in the local version of logfile_name 
     // in case reset_logfile() is called later so we can test whether it has changed
-    logfile_name = prog_config.logfile_name;
+    logfile_name = Glib::filename_from_utf8(prog_config.logfile_name);
   }
 }
 
@@ -504,13 +575,14 @@ MessageText::~MessageText(void) {
   
   logfile << "\n\nEnding fax log at " << date_description << '\n'
 	  << "***********************\n" << endl;
+  logfile.clear();
   logfile.close();
 }
 
 void MessageText::reset_logfile(void) {
 
   // check pre-conditions
-  if (logfile_name == prog_config.logfile_name) return; // no change!
+  if (logfile_name == Glib::filename_from_utf8(prog_config.logfile_name)) return; // no change!
 
   // proceed
   // first close the log file if required
@@ -528,6 +600,7 @@ void MessageText::reset_logfile(void) {
   
     logfile << "\n\nEnding fax log at " << date_description << '\n'
 	    << "***********************\n" << endl;
+    logfile.clear();
     logfile.close();
     // and now disconnect the old timer connection
     timer_slot_connection.disconnect();
@@ -536,7 +609,7 @@ void MessageText::reset_logfile(void) {
   // now open the new log file if required
   if (!prog_config.logfile_name.empty()) {
 
-    string temp(prog_config.logfile_name);
+    string temp(Glib::filename_from_utf8(prog_config.logfile_name));
     if (temp[0] != '/') temp.insert(0, starting_dirname); // provide an absolute path name
 
     logfile.open(temp.c_str(), ios::app | ios::out);
@@ -561,13 +634,13 @@ void MessageText::reset_logfile(void) {
       logfile << "\n\n***********************\n"
                  "Beginning fax log at " << date_description << '\n' << endl;
       // set a timeout to flush every one minute
-      timer_slot_connection = Gtk::Main::timeout.connect(SigC::slot(this,
-					 &MessageText::flush_logfile_timer_slot), LOGFILE_TIMER_INTERVAL);
+      timer_slot_connection = Glib::signal_timeout().connect(SigC::slot(*this,
+                                         &MessageText::flush_logfile_timer_slot), LOGFILE_TIMER_INTERVAL);
     }
   }
   // save the logfile name in the local version of logfile_name 
   // in case reset_logfile() is called again later
-  logfile_name = prog_config.logfile_name;
+  logfile_name = Glib::filename_from_utf8(prog_config.logfile_name);
 }
 
 void MessageText::cleanify(string& message) {
@@ -602,7 +675,17 @@ void MessageText::cleanify(string& message) {
 void MessageText::write_black_slot(const char* message) {
   string temp(message);
   cleanify(temp);
-  text.insert(temp);
+
+  bool scrolling = false;
+  if (get_vadjustment()->get_value() >=
+      (get_vadjustment()->get_upper() - get_vadjustment()->get_page_size() - 1e-12)) {
+    scrolling = true;
+  }
+
+  Glib::RefPtr<Gtk::TextBuffer> buffer_r = text.get_buffer();
+  buffer_r->insert(buffer_r->end(), temp);
+  if (scrolling) text.scroll_to_mark(buffer_r->create_mark("end", buffer_r->end()), 0.0);
+
   // now relay to standard output
   write(1, temp.c_str(), temp.size());
   // now log to file if required
@@ -612,46 +695,59 @@ void MessageText::write_black_slot(const char* message) {
 void MessageText::write_red_slot(const char* message) {
   string temp(message);
   cleanify(temp);
-  text.insert(red_context, temp);
+
+  bool scrolling = false;
+  if (get_vadjustment()->get_value() >=
+      (get_vadjustment()->get_upper() - get_vadjustment()->get_page_size() - 1e-12)) {
+    scrolling = true;
+  }
+
+  Glib::RefPtr<Gtk::TextBuffer> buffer_r = text.get_buffer();
+  buffer_r->insert_with_tag(buffer_r->end(), temp, red_tag_r);
+  if (scrolling) text.scroll_to_mark(buffer_r->create_mark("end", buffer_r->end()), 0.0);
+
   // now relay to standard error
   write(2, temp.c_str(), temp.size());
   // now log to file if required
   if (!prog_config.logfile_name.empty()) logfile << temp;
 }
 
-gint MessageText::flush_logfile_timer_slot(void) {
+bool MessageText::flush_logfile_timer_slot(void) {
   logfile.flush();
   return true; // we want a multi-shot timer
 }
 
 StatusLine::StatusLine(const int standard_size): 
-                                       Gtk::HBox(false, 0), status_label("Inactive"),
-                                       flexi_label("Press F1 for help") {
+                                       Gtk::HBox(false, 0), status_label(gettext("Inactive")),
+                                       help_label(gettext("Press F1 for help")) {
 
   set_border_width(2);
-  status_label.set_usize(standard_size * 7, standard_size - 4);
-  status_frame.set_shadow_type(GTK_SHADOW_IN);
+  status_label.set_size_request(standard_size * 7, standard_size - 4);
+  status_frame.set_shadow_type(Gtk::SHADOW_IN);
   status_frame.add(status_label);
-  pack_start(status_frame, false, false, 0);
+  pack_start(status_frame, true, true, 0);
   
-  flexi_label.set_usize(standard_size * 6, standard_size - 4);
-  flexi_frame.set_shadow_type(GTK_SHADOW_IN);
-  flexi_frame.add(flexi_label);
-  pack_end(flexi_frame, true, true, 0);
+  help_label.set_size_request(standard_size * 6, standard_size - 4);
+  help_frame.set_shadow_type(Gtk::SHADOW_IN);
+  help_frame.add(help_label);
+  pack_end(help_frame, true, true, 0);
+
+  show_all();
+}
+
+void StatusLine::set_colour(const Glib::RefPtr<Gtk::Style>& current_style_r) {
 
   // make the status label display in red
   // Note: we need to take a copy of the existing style, as the existing
   // style is shared by all the program's widgets.  If we operate
-  // directly on the Gtk::Style object returned (by pointer) by
-  // Gtk::Widget::get_style(), say with Gtk::Style::set_fg() or
-  // Gtk::Style::set_font(), we will alter it for everything!
-  Gtk::Style* new_style_p = status_label.get_style()->copy();
-  Gdk_Color red;
+  // directly on the Gtk::Style object returned (by Glib::RefPtr) by
+  // Gtk::Widget::get_style(), say with Gtk::Style::set_fg(),
+  // we will alter it for everything!
+  Glib::RefPtr<Gtk::Style> new_style_r(current_style_r->copy());
+  Gdk::Color red;
   red.set_rgb_p(0.9, 0, 0);
-  new_style_p->set_fg(GTK_STATE_NORMAL, red);
-  status_label.set_style(*new_style_p);
-
-  show_all();
+  new_style_r->set_fg(Gtk::STATE_NORMAL, red);
+  status_label.set_style(new_style_r);
 }
 
 int connect_to_stderr(void) {
